#!/bin/bash
# 20181018

# использование
# upos.sh init
# upos.sh ping|closeday dir_out prefix
# upos.sh pay|cancel|preauth amount dir_out prefix
# upos.sh postauth amount dir_out prefix rnn

# коды выхода
# 0 - успех
# 1 - устройство не найдено / не отвечает / ошибка выполнение операции, нужно смотреть *e
# 2 - нет исполняемого sb_pilot
# 3 - недостаточно аргументов
# 4 - неизвестное действие
# 5 - отсутсвуеть дирестория для сохранения результата
# 6 - не создались файлы данных sb_pilot

ACTION="$1"
AMOUNT="$2"

ERRORFILE="e"
PRINTFILE="p"
SBPATH="/home/terminal/atm/bin/pos/upos"

mkdir -p ${SBPATH} > /dev/null 2>&1

DN=$(dirname "$0")
cd ${SBPATH}

function sudo_execute() {
    echo terminal12 | sudo -S sh -c "$@"
}

function devcheck() {
    ${SBP} 26 2>&1 | grep return | sed "s/return://"
}

function writeresult() {
       FN="${1}"

    if [[ -f ./${ERRORFILE} ]]; then
        echo "TermNum=`sed "8q;d" ./${ERRORFILE}`" > ${FN}_p
        echo "ClientCard=`sed "2q;d" ./${ERRORFILE}`" >> ${FN}_p
        echo "ClientExpiryDate=`sed "3q;d" ./${ERRORFILE}`" >> ${FN}_p
        echo "AuthCode=`sed "4q;d" ./${ERRORFILE}`" >> ${FN}_p
        echo "CardName=`sed "6q;d" ./${ERRORFILE}`" >> ${FN}_p
        echo "CardType=`sed "7q;d" ./${ERRORFILE}`" >> ${FN}_p
        echo "MerchNum=`sed "14q;d" ./${ERRORFILE}`" >> ${FN}_p
        echo "RRN=`sed "10q;d" ./${ERRORFILE}`" >> ${FN}_p
        echo "TrxDate=`sed "9q;d" ./${ERRORFILE}`" >> ${FN}_p
        echo "Hash=`sed "11q;d" ./${ERRORFILE}`" >> ${FN}_p
        head -n1 ./${ERRORFILE} | cut -f1 -d"," > ${FN}_e
    fi

    if [[ -f ./${PRINTFILE} ]]; then
        iconv -f koi8-r ./${PRINTFILE} > ${FN}_c
    fi
}

function getstatus() {
    if [[ -f ./${ERRORFILE} ]]; then
        head -n1 ./${ERRORFILE} | cut -f1 -d","
    else
        exitstatus "6"
    fi
}

function writestatus() {
    FN="${2}"
    if [[ -f ./${ERRORFILE} ]]; then
        head -n1 ./${ERRORFILE} | cut -f1 -d"," > ${FN}_e
    else
        exitstatus "6"
    fi

    exitstatus ${1}
}

function exitstatus() {
    echo "sbpilotcode: ${1}"
    if [[ ${1} -gt "10" ]]; then
        code="1"
    else
        code="${1}"
    fi
    echo "exitcode: ${code}"
    exit ${code}
}

# init
if [[ "${ACTION}" == "init" ]]; then
    sudo_execute "chmod 775 /run/lock"
    sudo_execute "chown root.lock /run/lock"
    sudo_execute "chmod 775 /var/lock"
    sudo_execute "chown root.lock /var/lock"
    sudo_execute "rm -f /run/lock/sbrf-upos-main.lock"
    sudo_execute "rm -f /var/lock/sbrf-upos-main.lock"
    sudo_execute "rm -f /run/lock/sbrf-upos-pos-scheduler.lock"
    sudo_execute "rm -f /var/lock/sbrf-upos-pos-scheduler.lock"
    chmod 755 ${SBPATH}/sb_pilot
    chmod 755 ${SBPATH}/posScheduler

    devices=( usb-upos usb-079b_0028-if00 usb-VeriFone_Inc_Trident_USB_Device_1.1_0123456789ABCD-if00 usb-VeriFone_Verix_USB_Device_323-277-3449-if00 )

    initres="1"
    for device in ${devices[@]} ; do
        if [[ -h /dev/serial/by-id/${device} ]]; then
            rm -f ${SBPATH}/ttyS99
            ln -sf /dev/serial/by-id/${device} ${SBPATH}/ttyS99
            initres="0"
        fi
    done
    exitstatus ${initres}
fi

if [[ $# -lt 3 ]]; then
    exitstatus "3"
fi

which sb_pilot
if [[ $? -eq 0 ]]; then
    SBP="sb_pilot"
elif [[ -x ${SBPATH}/sb_pilot ]]; then
    SBP="${SBPATH}/sb_pilot"
else
    exitstatus "2"
fi

if [[ -z "${ACTION}" ]]; then
    exitstatus "3"
fi

if [[ "${ACTION}" == "postauth" && $# -lt 5 ]];then
    exitstatus "3"
fi

if [[ $# -gt 3 ]]; then
    filepath="${3}"
    fileprefix="${4}"
else
    filepath="${2}"
    fileprefix="${3}"
fi

if [[ ! -d ${filepath} ]];then
    exitstatus "5"
fi

pinres=$(devcheck)

if [[ -z ${pinres} ]]; then
    exitstatus "2"
fi

if [[ ${pinres} -eq 0 ]]; then
    case "${ACTION}" in
        pay)
        # платеж
        if [[ -z "${AMOUNT}" ]]; then
            exitstatus "3"
        else
            ${SBP} 1 ${AMOUNT} 0 > /dev/null 2>&1
            payres=$(getstatus)
            writeresult "${filepath}/${ACTION}_${fileprefix}"
            writestatus ${payres} "${filepath}/${ACTION}_${fileprefix}"
        fi
        ;;
        cancel)
        # возврат покупки
        if [[ -z "${AMOUNT}" ]]; then
            exitstatus "3"
        else
            ${SBP} 8 ${AMOUNT} 0 > /dev/null 2>&1
            payres=$(getstatus)
            writeresult "${filepath}/${ACTION}_${fileprefix}"
            writestatus ${payres} "${filepath}/${ACTION}_${fileprefix}"
        fi
        ;;

        preauth)
        # предавторизация
        if [[ -z "${AMOUNT}" ]]; then
            exitstatus "3"
        else
            ${SBP} 17 ${AMOUNT} 0 > /dev/null 2>&1
            payres=$(getstatus)
            writeresult "${filepath}/${ACTION}_${fileprefix}"
            writestatus ${payres} "${filepath}/${ACTION}_${fileprefix}"
        fi
        ;;

        postauth)
        # предавторизация
        if [[ -z "${AMOUNT}" ]]; then
            exitstatus "3"
        else
            ${SBP} 18 ${AMOUNT} 0 0 ${5} > /dev/null 2>&1
            payres=$(getstatus)
            writeresult "${filepath}/${ACTION}_${fileprefix}"
            writestatus ${payres} "${filepath}/${ACTION}_${fileprefix}"
        fi
        ;;

        closeday)
        # закрытие дня
        ${SBP} 7 > /dev/null 2>&1
        res=$(getstatus)
        writeresult "${filepath}/${ACTION}_${fileprefix}"
        writestatus ${res} "${filepath}/${ACTION}_${fileprefix}"
        ;;

        info)
        # запрос информации
        ${SBP} 36 > /dev/null 2>&1
        res=$(getstatus)
        writeresult "${filepath}/${ACTION}_${fileprefix}"
        writestatus ${res} "${filepath}/${ACTION}_${fileprefix}"
        ;;

        ping)
        writestatus ${pinres} "${filepath}/${ACTION}_${fileprefix}"
        ;;

        *)
        echo "Usage: $0 [ping|pay|cancel|closeday] [amount]"
        exitstatus "4"
        ;;
    esac
else
    writestatus ${pinres} "${filepath}/${ACTION}_${fileprefix}"
fi

exit 0
